@@ -7,6 +7,7 @@ Tests for various parts of L{twisted.web}.
 
 import os
 import zlib
+from typing import List
 
 from zope.interface import implementer
 from zope.interface.verify import verifyObject
@@ -14,6 +15,7 @@ from zope.interface.verify import verifyObject
 from twisted.python import reflect, failure
 from twisted.python.compat import unichr
 from twisted.python.filepath import FilePath
+from twisted.python.compat import iterbytes
 from twisted.trial import unittest
 from twisted.internet import reactor, interfaces
 from twisted.internet.address import IPv4Address, IPv6Address
@@ -24,8 +26,9 @@ from twisted.web import iweb, http, error
 from twisted.web.test.requesthelper import DummyChannel, DummyRequest
 from twisted.web.static import Data
 from twisted.logger import globalLogPublisher, LogLevel
-from twisted.test.proto_helpers import EventLoggingObserver
-
+from twisted.test.proto_helpers import EventLoggingObserver, StringTransport
+from twisted.web.resource import Resource
+from twisted.web.server import NOT_DONE_YET, Request, Site
 
 class ResourceTests(unittest.TestCase):
     def testListEntities(self):
@@ -772,8 +775,8 @@ class RequestTests(unittest.TestCase):
 
     def test_retrieveNonExistentSession(self):
         """
-        L{Request.getSession} generates a new session if the relevant cookie is
-        set in the incoming request.
+        L{Request.getSession} generates a new session if the session ID
+        advertised in the cookie from the incoming request is not found.
         """
         site = server.Site(resource.Resource())
         d = DummyChannel()
@@ -789,6 +792,51 @@ class RequestTests(unittest.TestCase):
         self.assertNotIn(b"does-not-exist", request.cookies[0])
 
 
+    def test_getSessionExpired(self):
+        """
+        L{Request.getSession} generates a new session when the previous
+        session has expired.
+        """
+        clock = Clock()
+        site = server.Site(resource.Resource())
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.site = site
+        request.sitepath = []
+
+        def sessionFactoryWithClock(site, uid):
+            """
+            Forward to normal session factory, but inject the clock.
+
+            @param site: The site on which the session is created.
+            @type site: L{server.Site}
+
+            @param uid: A unique identifier for the session.
+            @type uid: C{bytes}
+
+            @return: A newly created session.
+            @rtype: L{server.Session}
+            """
+            session = sessionFactory(site, uid)
+            session._reactor = clock
+            return session
+
+        # The site is patch to allow injecting a clock to the session.
+        sessionFactory = site.sessionFactory
+        site.sessionFactory = sessionFactoryWithClock
+
+        initialSession = request.getSession()
+
+        # When the session is requested after the session timeout,
+        # no error is raised and a new session is returned.
+        clock.advance(sessionFactory.sessionTimeout)
+        newSession = request.getSession()
+        self.addCleanup(newSession.expire)
+
+        self.assertIsNot(initialSession, newSession)
+        self.assertNotEqual(initialSession.uid, newSession.uid)
+
+
     def test_OPTIONSStar(self):
         """
         L{Request} handles OPTIONS * requests by doing a fast-path return of
@@ -1393,7 +1441,7 @@ class AccessLogTestsMixin(object):
 
         self.assertEqual(
             # Client IP
-            b'"1.2.3.4" '
+            b'1.2.3.4 '
             # Some blanks we never fill in
             b'- - '
             # The current time (circa 1234567890)
@@ -1486,7 +1534,7 @@ class CombinedLogFormatterTests(unittest.TestCase):
 
         line = http.combinedLogFormatter(timestamp, request)
         self.assertEqual(
-            u'"evil x-forwarded-for \\x80" - - [13/Feb/2009:23:31:30 +0000] '
+            u'evil x-forwarded-for \\x80 - - [13/Feb/2009:23:31:30 +0000] '
             u'"POS\\x81 /dummy HTTP/1.0" 123 - "evil \\x83" "evil \\x84"',
             line)
 
@@ -1504,7 +1552,7 @@ class CombinedLogFormatterTests(unittest.TestCase):
 
         line = http.combinedLogFormatter(timestamp, request)
         self.assertEqual(
-            u'"::1" - - [13/Feb/2009:23:31:30 +0000] '
+            u'::1 - - [13/Feb/2009:23:31:30 +0000] '
             u'"GET /dummy HTTP/1.0" 123 - "-" "-"',
             line)
 
@@ -1528,7 +1576,7 @@ class CombinedLogFormatterTests(unittest.TestCase):
         request.client = UnknowableAddress()
 
         line = http.combinedLogFormatter(timestamp, request)
-        self.assertTrue(line.startswith(u'"-" '))
+        self.assertTrue(line.startswith(u'- '))
 
 
 
@@ -1618,7 +1666,7 @@ class LogEscapingTests(unittest.TestCase):
         self.site._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
             25, 'Oct', 2004, 12, 31, 59)
         self.assertLogs(
-            b'"1.2.3.4" - - [25/Oct/2004:12:31:59 +0000] '
+            b'1.2.3.4 - - [25/Oct/2004:12:31:59 +0000] '
             b'"GET /dummy HTTP/1.0" 123 - "-" "-"\n')
 
 
@@ -1630,7 +1678,7 @@ class LogEscapingTests(unittest.TestCase):
             25, 'Oct', 2004, 12, 31, 59)
         self.request.method = b'G"T'
         self.assertLogs(
-            b'"1.2.3.4" - - [25/Oct/2004:12:31:59 +0000] '
+            b'1.2.3.4 - - [25/Oct/2004:12:31:59 +0000] '
             b'"G\\"T /dummy HTTP/1.0" 123 - "-" "-"\n')
 
 
@@ -1642,7 +1690,7 @@ class LogEscapingTests(unittest.TestCase):
             25, 'Oct', 2004, 12, 31, 59)
         self.request.uri = b'/dummy"withquote'
         self.assertLogs(
-            b'"1.2.3.4" - - [25/Oct/2004:12:31:59 +0000] '
+            b'1.2.3.4 - - [25/Oct/2004:12:31:59 +0000] '
             b'"GET /dummy\\"withquote HTTP/1.0" 123 - "-" "-"\n')
 
 
@@ -1654,7 +1702,7 @@ class LogEscapingTests(unittest.TestCase):
             25, 'Oct', 2004, 12, 31, 59)
         self.request.clientproto = b'HT"P/1.0'
         self.assertLogs(
-            b'"1.2.3.4" - - [25/Oct/2004:12:31:59 +0000] '
+            b'1.2.3.4 - - [25/Oct/2004:12:31:59 +0000] '
             b'"GET /dummy HT\\"P/1.0" 123 - "-" "-"\n')
 
 
@@ -1669,7 +1717,7 @@ class LogEscapingTests(unittest.TestCase):
             b'referer',
             b'http://malicious" ".website.invalid')
         self.assertLogs(
-            b'"1.2.3.4" - - [25/Oct/2004:12:31:59 +0000] '
+            b'1.2.3.4 - - [25/Oct/2004:12:31:59 +0000] '
             b'"GET /dummy HTTP/1.0" 123 - '
             b'"http://malicious\\" \\".website.invalid" "-"\n')
 
@@ -1684,7 +1732,7 @@ class LogEscapingTests(unittest.TestCase):
         self.request.requestHeaders.addRawHeader(b'user-agent',
                                                  b'Malicious Web" Evil')
         self.assertLogs(
-            b'"1.2.3.4" - - [25/Oct/2004:12:31:59 +0000] '
+            b'1.2.3.4 - - [25/Oct/2004:12:31:59 +0000] '
             b'"GET /dummy HTTP/1.0" 123 - "-" "Malicious Web\\" Evil"\n')
 
 
@@ -1753,3 +1801,78 @@ class ExplicitHTTPFactoryReactor(unittest.TestCase):
         from twisted.internet import reactor
         factory = http.HTTPFactory()
         self.assertIs(factory._reactor, reactor)
+
+
+class QueueResource(Resource):
+    """
+    Add all requests to an internal queue,
+    without responding to the requests.
+    You can access the requests from the queue and handle their response.
+    """
+
+    isLeaf = True
+
+    def __init__(self) -> None:
+        super().__init__()
+        self.dispatchedRequests: List[Request] = []
+
+    def render_GET(self, request: Request) -> int:
+        self.dispatchedRequests.append(request)
+        return NOT_DONE_YET
+
+
+class TestRFC9112Section932(unittest.TestCase):
+    """
+    Verify that HTTP/1.1 request ordering is preserved.
+    """
+
+    def test_multipleRequestsInOneSegment(self) -> None:
+        """
+        Twisted MUST NOT respond to a second HTTP/1.1 request while the first
+        is still pending.
+        """
+        qr = QueueResource()
+        site = Site(qr)
+        proto = site.buildProtocol(None)
+        serverTransport = StringTransport()
+        proto.makeConnection(serverTransport)
+        proto.dataReceived(
+            b"GET /first HTTP/1.1\r\nHost: a\r\n\r\n"
+            b"GET /second HTTP/1.1\r\nHost: a\r\n\r\n"
+        )
+        # The TCP data contains 2 requests,
+        # but only 1 request was dispatched,
+        # as the first request was not yet finalized.
+        self.assertEqual(len(qr.dispatchedRequests), 1)
+        # The first request is finalized and the
+        # second request is dispatched right away.
+        qr.dispatchedRequests[0].finish()
+        self.assertEqual(len(qr.dispatchedRequests), 2)
+
+    def test_multipleRequestsInDifferentSegments(self) -> None:
+        """
+        Twisted MUST NOT respond to a second HTTP/1.1 request while the first
+        is still pending, even if the second request is received in a separate
+        TCP package.
+        """
+        qr = QueueResource()
+        site = Site(qr)
+        proto = site.buildProtocol(None)
+        serverTransport = StringTransport()
+        proto.makeConnection(serverTransport)
+        raw_data = (
+            b"GET /first HTTP/1.1\r\nHost: a\r\n\r\n"
+            b"GET /second HTTP/1.1\r\nHost: a\r\n\r\n"
+        )
+        # Just go byte by byte for the extreme case in which each byte is
+        # received in a separate TCP package.
+        for chunk in iterbytes(raw_data):
+            proto.dataReceived(chunk)
+        # The TCP data contains 2 requests,
+        # but only 1 request was dispatched,
+        # as the first request was not yet finalized.
+        self.assertEqual(len(qr.dispatchedRequests), 1)
+        # The first request is finalized and the
+        # second request is dispatched right away.
+        qr.dispatchedRequests[0].finish()
+        self.assertEqual(len(qr.dispatchedRequests), 2)
